"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsentContract = void 0;
const objects_1 = require("../interfaces/objects");
const abi_1 = require("../interfaces/objects/abi");
const index_js_1 = require("../interfaces/objects/index.js");
const objects_2 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
const neverthrow_result_utils_1 = require("neverthrow-result-utils");
let ConsentContract = class ConsentContract {
    constructor(providerOrSigner, contractAddress, cryptoUtils) {
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
        this.cryptoUtils = cryptoUtils;
        this.filters = {
            Transfer: (fromAddress, toAddress) => {
                return this.contract.filters.Transfer(fromAddress, toAddress);
            },
            RequestForData: (ownerAddress) => {
                return this.contract.filters.RequestForData(ownerAddress);
            },
        };
        this.contract = new ethers_1.ethers.Contract(contractAddress, abi_1.ContractsAbis.ConsentAbi.abi, providerOrSigner);
    }
    getContractAddress() {
        return this.contractAddress;
    }
    optIn(tokenId, agreementFlags) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.optIn(tokenId, agreementFlags), (e) => {
            return new objects_2.ConsentContractError("Unable to call optIn()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for optIn() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    // TODO: add data permissions param
    encodeOptIn(tokenId, agreementFlags) {
        return (0, objects_2.HexString)(this.contract.interface.encodeFunctionData("optIn", [
            tokenId,
            agreementFlags,
        ]));
    }
    restrictedOptIn(tokenId, agreementFlags, signature) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.restrictedOptIn(tokenId, agreementFlags, signature), (e) => {
            return new objects_2.ConsentContractError("Unable to call restrictedOptIn()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for restrictedOptIn() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    encodeRestrictedOptIn(tokenId, signature, agreementFlags) {
        return (0, objects_2.HexString)(this.contract.interface.encodeFunctionData("restrictedOptIn", [
            tokenId,
            agreementFlags,
            signature,
        ]));
    }
    anonymousRestrictedOptIn(tokenId, agreementFlags, signature) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.anonymousRestrictedOptIn(tokenId, agreementFlags, signature), (e) => {
            return new objects_2.ConsentContractError("Unable to call anonymousRestrictedOptIn()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for anonymousRestrictedOptIn() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    encodeAnonymousRestrictedOptIn(tokenId, signature, agreementFlags) {
        return (0, objects_2.HexString)(this.contract.interface.encodeFunctionData("anonymousRestrictedOptIn", [
            tokenId,
            agreementFlags,
            signature,
        ]));
    }
    optOut(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.optOut(tokenId), (e) => {
            return new objects_2.ConsentContractError("Unable to call optOut()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for optOut() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    encodeOptOut(tokenId) {
        return (0, objects_2.HexString)(this.contract.interface.encodeFunctionData("optOut", [tokenId]));
    }
    agreementFlags(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.agreementFlagsArray(tokenId), (e) => {
            return new objects_2.ConsentContractError("Unable to call agreementFlagsArray()", e.reason, e);
        });
    }
    getMaxCapacity() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.maxCapacity(), (e) => {
            return new objects_2.ConsentContractError("Unable to call maxCapacity()", e.reason, e);
        }).map((bigCapacity) => {
            return bigCapacity.toNumber();
        });
    }
    updateMaxCapacity(maxCapacity) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.updateMaxCapacity(maxCapacity), (e) => {
            return new objects_2.ConsentContractError("Unable to call updateMaxCapacity()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for updateMaxCapacity() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    requestForData(ipfsCID) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.requestForData(ipfsCID), (e) => {
            return new objects_2.ConsentContractError("Unable to call requestForData()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for requestForData() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    // Returns the address Consent contract contract's owner
    // Note that the address on index 0 of the DEFAULT_ADMIN_ROLE members is the contract owner
    getConsentOwner() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(index_js_1.ConsentRoles.DEFAULT_ADMIN_ROLE, 0), (e) => {
            return new objects_2.ConsentContractError("Unable to call getRoleMember()", e.reason, e);
        });
    }
    // Returns an array of address that has the DEFAULT_ADMIN_ROLE on the Consent contract
    // Note that the address on index 0 is the contract owner
    getDefaultAdminRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(index_js_1.ConsentRoles.DEFAULT_ADMIN_ROLE), (e) => {
            return new objects_2.ConsentContractError("Unable to call getDefaultAdminRoleMembers()", e.reason, e);
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(index_js_1.ConsentRoles.DEFAULT_ADMIN_ROLE, index), (e) => {
                    return new objects_2.ConsentContractError("Unable to call getRoleMember()", e.reason, e);
                });
            }));
        });
    }
    getSignerRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(index_js_1.ConsentRoles.SIGNER_ROLE), (e) => {
            return new objects_2.ConsentContractError("Unable to call getSignerRoleMembers()", e.reason, e);
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(index_js_1.ConsentRoles.PAUSER_ROLE, index), (e) => {
                    return new objects_2.ConsentContractError("Unable to call getRoleMember() for SIGNER_ROLE", e.reason, e);
                });
            }));
        });
    }
    getPauserRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(index_js_1.ConsentRoles.PAUSER_ROLE), (e) => {
            return new objects_2.ConsentContractError("Unable to call getPauserRoleMembers()", e.reason, e);
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(index_js_1.ConsentRoles.PAUSER_ROLE, index), (e) => {
                    return new objects_2.ConsentContractError("Unable to call getRoleMember() for PAUSER_ROLE", e.reason, e);
                });
            }));
        });
    }
    getRequesterRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(index_js_1.ConsentRoles.REQUESTER_ROLE), (e) => {
            return new objects_2.ConsentContractError("Unable to call getRequesterRoleMembers()", e.reason, e);
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(index_js_1.ConsentRoles.REQUESTER_ROLE, index), (e) => {
                    return new objects_2.ConsentContractError("Unable to call getRoleMember() for REQUESTER_ROLE", e.reason, e);
                });
            }));
        });
    }
    balanceOf(address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.balanceOf(address), (e) => {
            return new objects_2.ConsentContractError("Unable to call balanceOf()", e.reason, e);
        }).map((numberOfTokens) => {
            return numberOfTokens.toNumber();
        });
    }
    ownerOf(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.ownerOf(tokenId), (e) => {
            return new objects_2.ConsentContractError("Unable to call ownerOf()", e.reason, e);
        });
    }
    tokenURI(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.tokenURI(tokenId), (e) => {
            return new objects_2.ConsentContractError("Unable to call tokenURI()", e.reason, e);
        }).orElse((error) => {
            // The contract reverts with this message if tokenId does not exist
            if (error.reason === "ERC721: operator query for nonexistent token") {
                return (0, neverthrow_1.ok)(null);
            }
            return (0, neverthrow_1.err)(error);
        });
    }
    queryFilter(eventFilter, fromBlock, toBlock) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.queryFilter(eventFilter, fromBlock, toBlock), (e) => {
            return new objects_2.ConsentContractError("Unable to call queryFilter()", e.reason, e);
        });
    }
    getConsentToken(tokenId) {
        // Get the agreement flags of the user's current consent token
        return neverthrow_result_utils_1.ResultUtils.combine([
            this.ownerOf(tokenId),
            this.agreementFlags(tokenId),
        ]).andThen(([ownerAddress, agreementFlags]) => {
            return (0, neverthrow_1.okAsync)(new objects_2.ConsentToken(this.contractAddress, ownerAddress, tokenId, new objects_2.DataPermissions(agreementFlags)));
        });
    }
    addDomain(domain) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.addDomain(domain), (e) => {
            return new objects_2.ConsentContractError("Unable to call addDomain()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for addDomain() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    removeDomain(domain) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.removeDomain(domain), (e) => {
            return new objects_2.ConsentContractError("Unable to call removeDomain()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for removeDomain() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    getDomains() {
        return neverthrow_1.ResultAsync.fromPromise(
        // returns array of domains
        this.contract.getDomains(), (e) => {
            return new objects_2.ConsentContractError("Unable to call getDomains()", e.reason, e);
        });
    }
    getRequestForDataListByRequesterAddress(requesterAddress, fromBlock, toBlock) {
        return this.queryFilter(this.filters.RequestForData(requesterAddress), fromBlock, toBlock).andThen((logsEvents) => {
            return neverthrow_result_utils_1.ResultUtils.combine(logsEvents.map((logEvent) => {
                var _a, _b;
                return (0, neverthrow_1.okAsync)(new objects_2.RequestForData(this.getContractAddress(), (_a = logEvent.args) === null || _a === void 0 ? void 0 : _a.requester, (_b = logEvent.args) === null || _b === void 0 ? void 0 : _b.ipfsCID, (0, objects_2.BlockNumber)(logEvent.blockNumber)));
            }));
        });
    }
    disableOpenOptIn() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.disableOpenOptIn(), (e) => {
            return new objects_2.ConsentContractError("Unable to call disableOpenOptIn()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for disableOpenOptIn() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    enableOpenOptIn() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.enableOpenOptIn(), (e) => {
            return new objects_2.ConsentContractError("Unable to call enableOpenOptIn()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for enableOpenOptIn() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    baseURI() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.baseURI(), (e) => {
            return new objects_2.ConsentContractError("Unable to call baseURI()", e.reason, e);
        });
    }
    setBaseURI(baseUri) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.setBaseURI(baseUri), (e) => {
            return new objects_2.ConsentContractError("Unable to call setBaseURI()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for setBaseURI() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    hasRole(role, address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.hasRole(index_js_1.ConsentRoles[role], address), (e) => {
            return new objects_2.ConsentContractError("Unable to call hasRole()", e.reason, e);
        });
    }
    grantRole(role, address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.grantRole(index_js_1.ConsentRoles[role], address), (e) => {
            return new objects_2.ConsentContractError("Unable to call grantRole()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for grantRole() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    revokeRole(role, address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.revokeRole(index_js_1.ConsentRoles[role], address), (e) => {
            return new objects_2.ConsentContractError("Unable to call revokeRole()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for revokeRole() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    renounceRole(role, address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.renounceRole(index_js_1.ConsentRoles[role], address), (e) => {
            return new objects_2.ConsentContractError("Unable to call renounceRole()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for renounceRole() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    getQueryHorizon() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.queryHorizon(), (e) => {
            return new objects_2.ConsentContractError("Unable to call queryHorizon()", e.reason, e);
        }).map((queryHorizon) => (0, objects_2.BlockNumber)(queryHorizon.toNumber()));
    }
    setQueryHorizon(blockNumber) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.setQueryHorizon(blockNumber), (e) => {
            return new objects_2.ConsentContractError("Unable to call setQueryHorizon()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_2.ConsentContractError("Wait for setQueryHorizon() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    // Get the number of opted in addresses
    totalSupply() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.totalSupply(), (e) => {
            return new objects_2.ConsentContractError("Unable to call totalSupply()", e.reason, e);
        }).map((totalSupply) => totalSupply.toNumber());
    }
    openOptInDisabled() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.openOptInDisabled(), (e) => {
            return new objects_2.ConsentContractError("Unable to call openOptInDisabled()", "Unknown", e);
        });
    }
    // Marketplace functions
    getMaxTags() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.maxTags(), (e) => {
            return new objects_2.ConsentContractError("Unable to call openOptInDisabled()", "Unknown", e);
        }).map((num) => {
            return num.toNumber();
        });
    }
    getNumberOfStakedTags() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getNumberOfStakedTags(), (e) => {
            return new objects_2.ConsentContractError("Unable to call getNumberOfStakedTags()", "Unknown", e);
        }).map((num) => {
            return num.toNumber();
        });
    }
    getTagArray() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getTagArray(), (e) => {
            return new objects_2.ConsentContractError("Unable to call getTagArray()", "Unknown", e);
        }).map((tags) => {
            return tags.map((tag) => {
                return new index_js_1.Tag(tag.slot ? (0, objects_2.BigNumberString)(tag.slot.toString()) : null, tag.tag, tag.staker);
            });
        });
    }
    newGlobalTag(tag, newStakeAmount) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.newGlobalTag(tag, newStakeAmount), (e) => {
            return new objects_2.ConsentContractError("Unable to call newGlobalTag()", e.reason, e);
        }).map((tx) => {
            return new objects_1.WrappedTransactionResponse(tx);
        });
    }
    newLocalTagUpstream(tag, newStakeAmount, existingStakeAmount) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.newLocalTagUpstream(tag, newStakeAmount, existingStakeAmount), (e) => {
            return new objects_2.ConsentContractError("Unable to call newLocalTagUpstream()", e.reason, e);
        }).map((tx) => {
            return new objects_1.WrappedTransactionResponse(tx);
        });
    }
    newLocalTagDownstream(tag, existingStakeAmount, newStakeAmount) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.newLocalTagDownstream(tag, existingStakeAmount, newStakeAmount), (e) => {
            return new objects_2.ConsentContractError("Unable to call newLocalTagDownstream()", e.reason, e);
        }).map((tx) => {
            return new objects_1.WrappedTransactionResponse(tx);
        });
    }
    replaceExpiredListing(tag, stakeAmount) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.replaceExpiredListing(tag, stakeAmount), (e) => {
            return new objects_2.ConsentContractError("Unable to call replaceExpiredListing()", e.reason, e);
        }).map((tx) => {
            return new objects_1.WrappedTransactionResponse(tx);
        });
    }
    removeListing(tag) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.removeListing(tag), (e) => {
            return new objects_2.ConsentContractError("Unable to call removeListing()", e.reason, e);
        }).map((tx) => {
            return new objects_1.WrappedTransactionResponse(tx);
        });
    }
    getSignature(values) {
        const types = ["address", "uint256"];
        return this.cryptoUtils.getSignature(this.providerOrSigner, types, values);
    }
};
ConsentContract = __decorate([
    (0, inversify_1.injectable)()
], ConsentContract);
exports.ConsentContract = ConsentContract;
//# sourceMappingURL=ConsentContract.js.map