"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CrumbsContract = void 0;
const objects_1 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
const abi_1 = require("../interfaces/objects/abi");
let CrumbsContract = class CrumbsContract {
    constructor(providerOrSigner, contractAddress) {
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
        this.contract = new ethers_1.ethers.Contract(contractAddress, abi_1.ContractsAbis.CrumbsAbi.abi, providerOrSigner);
    }
    addressToCrumbId(accountAddress) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.addressToCrumbId(accountAddress), (e) => {
            return new objects_1.CrumbsContractError("Unable to call addressToCrumbId()", e.reason, e);
        }).map((tokenId) => {
            // The contract returns 0 for an address that does not have a Crumb Id
            // Handle by returning null
            if (tokenId == BigInt(0)) {
                return null;
            }
            return tokenId;
        });
    }
    tokenURI(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.tokenURI(tokenId), (e) => {
            return new objects_1.CrumbsContractError("Unable to call tokenURI()", e.reason, e);
        }).orElse((error) => {
            // The contract reverts with this message if tokenId does not exist
            if (error.reason === "ERC721: operator query for nonexistent token") {
                return (0, neverthrow_1.ok)(null);
            }
            return (0, neverthrow_1.err)(error);
        });
    }
    createCrumb(crumbId, tokenUri, contractOverrides) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.createCrumb(crumbId, tokenUri), (e) => {
            // No error handling needed, any reverts from function call should return the reason
            return new objects_1.CrumbsContractError("Unable to call createCrumb()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.CrumbsContractError("Wait for createCrumb() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    encodeCreateCrumb(crumbId, crumbContent) {
        return (0, objects_1.HexString)(this.contract.interface.encodeFunctionData("createCrumb", [
            crumbId,
            crumbContent,
        ]));
    }
    burnCrumb(crumbId, contractOverrides) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.burnCrumb(crumbId, contractOverrides), (e) => {
            // No error handling needed, any reverts from function call should return the reason
            return new objects_1.CrumbsContractError("Unable to call burnCrumb()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.CrumbsContractError("Wait for burnCrumb() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    encodeBurnCrumb(crumbId) {
        return (0, objects_1.HexString)(this.contract.interface.encodeFunctionData("burnCrumb", [crumbId]));
    }
    updateTokenURI(crumbId, tokenURI) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.updateCrumb(crumbId, tokenURI), (e) => {
            // No error handling needed, any reverts from function call should return the reason
            return new objects_1.CrumbsContractError("Unable to call updateCrumbId()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.CrumbsContractError("Wait for updateCrumbId() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    getContract() {
        return this.contract;
    }
};
CrumbsContract = __decorate([
    (0, inversify_1.injectable)()
], CrumbsContract);
exports.CrumbsContract = CrumbsContract;
//# sourceMappingURL=CrumbsContract.js.map