"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ERC721RewardContract = void 0;
const ERewardRoles_js_1 = require("../interfaces/enums/ERewardRoles.js");
const abi_1 = require("../interfaces/objects/abi");
const objects_1 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
const neverthrow_result_utils_1 = require("neverthrow-result-utils");
let ERC721RewardContract = class ERC721RewardContract {
    constructor(providerOrSigner, contractAddress) {
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
        this.filters = {
            Transfer: (fromAddress, toAddress) => {
                return this.contract.filters.Transfer(fromAddress, toAddress);
            },
        };
        this.contract = new ethers_1.ethers.Contract(contractAddress, abi_1.ContractsAbis.ERC721Reward.abi, providerOrSigner);
        this.contractFactory = new ethers_1.ethers.ContractFactory(abi_1.ContractsAbis.ERC721Reward.abi, abi_1.ContractsAbis.ERC721Reward.bytecode, providerOrSigner);
    }
    getContractAddress() {
        return this.contractAddress;
    }
    getOwner() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(ERewardRoles_js_1.ERewardRoles.DEFAULT_ADMIN_ROLE, 0), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call getRoleMember()", e.reason, e);
        });
    }
    // Returns an array of address that has the DEFAULT_ADMIN_ROLE on the Reward contract
    // Note that the address on index 0 is the contract owner
    getDefaultAdminRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(ERewardRoles_js_1.ERewardRoles.DEFAULT_ADMIN_ROLE), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call getDefaultAdminRoleMembers()", e.reason, e);
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(ERewardRoles_js_1.ERewardRoles.DEFAULT_ADMIN_ROLE, index), (e) => {
                    return new objects_1.ERC721RewardContractError("Unable to call getRoleMember()", e.reason, e);
                });
            }));
        });
    }
    getMinterRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(ERewardRoles_js_1.ERewardRoles.MINTER_ROLE), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call getSignerRoleMembers()", e.reason, e);
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(ERewardRoles_js_1.ERewardRoles.MINTER_ROLE, index), (e) => {
                    return new objects_1.ERC721RewardContractError("Unable to call getRoleMember() for MINTER_ROLE", e.reason, e);
                });
            }));
        });
    }
    baseURI() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.baseURI(), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call baseURI()", e.reason, e);
        });
    }
    setBaseURI(baseUri) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.setBaseURI(baseUri), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call setBaseURI()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.ERC721RewardContractError("Wait for setBaseURI() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    balanceOf(address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.balanceOf(address), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call balanceOf()", e.reason, e);
        }).map((numberOfTokens) => {
            return numberOfTokens.toNumber();
        });
    }
    ownerOf(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.ownerOf(tokenId), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call ownerOf()", e.reason, e);
        });
    }
    tokenURI(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.tokenURI(tokenId), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call tokenURI()", e.reason, e);
        }).orElse((error) => {
            // The contract reverts with this message if tokenId does not exist
            if (error.reason === "ERC721: operator query for nonexistent token") {
                return (0, neverthrow_1.ok)(null);
            }
            return (0, neverthrow_1.err)(error);
        });
    }
    hasRole(role, address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.hasRole(ERewardRoles_js_1.ERewardRoles[role], address), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call hasRole()", e.reason, e);
        });
    }
    grantRole(role, address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.grantRole(ERewardRoles_js_1.ERewardRoles[role], address), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call grantRole()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.ERC721RewardContractError("Wait for grantRole() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    revokeRole(role, address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.revokeRole(ERewardRoles_js_1.ERewardRoles[role], address), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call revokeRole()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.ERC721RewardContractError("Wait for revokeRole() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    renounceRole(role, address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.renounceRole(ERewardRoles_js_1.ERewardRoles[role], address), (e) => {
            return new objects_1.ERC721RewardContractError("Unable to call renounceRole()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.ERC721RewardContractError("Wait for renounceRole() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    getContract() {
        return this.contract;
    }
};
ERC721RewardContract = __decorate([
    (0, inversify_1.injectable)()
], ERC721RewardContract);
exports.ERC721RewardContract = ERC721RewardContract;
//# sourceMappingURL=ERC721RewardContract.js.map