"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RewardsContractFactory = void 0;
const abi_1 = require("../interfaces/objects/abi");
const objects_1 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
const GasUtils_1 = require("../implementations/GasUtils");
let RewardsContractFactory = class RewardsContractFactory {
    constructor(providerOrSigner, rewardType) {
        this.providerOrSigner = providerOrSigner;
        this.rewardType = rewardType;
        // Set the correct contract factory based on rewardTypeToDeploy
        this.contractFactory = new ethers_1.ethers.ContractFactory(abi_1.ContractsAbis.ERC721Reward.abi, abi_1.ContractsAbis.ERC721Reward.bytecode, providerOrSigner);
        this.rewardTypeToDeploy = rewardType;
    }
    // function to deploy a new ERC721 reward contract
    deployERC721Reward(name, symbol, baseURI, overrides = null) {
        return GasUtils_1.GasUtils.getGasFee(this.providerOrSigner).andThen((gasFee) => {
            return neverthrow_1.ResultAsync.fromPromise(this.contractFactory.deploy(name, symbol, baseURI, {
                ...gasFee,
                ...overrides,
            }), (e) => {
                return new objects_1.RewardsFactoryError("Failed to deploy contract", e.reason, e);
            }).andThen((contract) => {
                return neverthrow_1.ResultAsync.fromPromise(contract.deployTransaction.wait(), (e) => {
                    return new objects_1.RewardsFactoryError("Failed to wait() for contract deployment", e.reason, e);
                }).map((receipt) => {
                    return (0, objects_1.EVMContractAddress)(receipt.contractAddress);
                });
            });
        });
    }
    estimateGasToDeployERC721Contract(name, symbol, baseURI) {
        return neverthrow_1.ResultAsync.fromPromise(this.providerOrSigner.estimateGas(this.contractFactory.getDeployTransaction(name, symbol, baseURI)), (e) => {
            return new objects_1.RewardsFactoryError("Failed to wait() for contract deployment", e.reason, e);
        }).map((estimatedGas) => {
            // Increase estimated gas buffer by 20%
            return estimatedGas.mul(120).div(100);
        });
    }
};
RewardsContractFactory = __decorate([
    (0, inversify_1.injectable)()
], RewardsContractFactory);
exports.RewardsContractFactory = RewardsContractFactory;
//# sourceMappingURL=RewardsContractFactory.js.map