"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SiftContract = void 0;
const objects_1 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
const abi_1 = require("../interfaces/objects/abi");
let SiftContract = class SiftContract {
    constructor(providerOrSigner, contractAddress) {
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
        this.contract = new ethers_1.ethers.Contract(contractAddress, abi_1.ContractsAbis.SiftAbi.abi, providerOrSigner);
    }
    checkURL(domain) {
        // Returns the tokenURI or string
        // eg. 'www.sift.com/VERIFIED', 'www.sift.com/MALICIOUS' or 'NOT VERIFIED'
        return neverthrow_1.ResultAsync.fromPromise(this.contract.checkURL(domain), (e) => {
            return new objects_1.SiftContractError("Unable to call checkURL()", e.reason, e);
        });
    }
    verifyURL(domain) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.verifyURL(domain), (e) => {
            // No error handling needed, any reverts from function call should return the reason
            return new objects_1.SiftContractError("Unable to call verifyURL()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.SiftContractError("Wait for verifyURL() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    maliciousURL(domain) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.maliciousURL(domain), (e) => {
            // No error handling needed, any reverts from function call should return the reason
            return new objects_1.SiftContractError("Unable to call maliciousURL()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.SiftContractError("Wait for maliciousURL() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    setBaseURI(baseUri) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.setBaseURI(baseUri), (e) => {
            return new objects_1.SiftContractError("Unable to call setBaseURI()", e.reason, e);
        })
            .andThen((tx) => {
            return neverthrow_1.ResultAsync.fromPromise(tx.wait(), (e) => {
                return new objects_1.SiftContractError("Wait for setBaseURI() failed", "Unknown", e);
            });
        })
            .map(() => { });
    }
    getContract() {
        return this.contract;
    }
};
SiftContract = __decorate([
    (0, inversify_1.injectable)()
], SiftContract);
exports.SiftContract = SiftContract;
//# sourceMappingURL=SiftContract.js.map